part of disposable;

// TODO: Implement RefCountDisposable - https://github.com/Reactive-Extensions/RxJS/blob/master/rx.js
// TODO: Implement ScheduledDisposable - https://github.com/Reactive-Extensions/RxJS/blob/master/rx.js

/// Provides a stand-alone implementation of ensuring clean-up callback which doesn't require sub classing
class Disposable implements IFutureDisposable {
  static dynamic noop = () { };
  static dynamic noop1 = (dynamic value) { };
  final DisposableCallback _callback;
  final DisposableOnData _onData;
  bool _isDisposed = false;
  bool get isDisposed => _isDisposed;
  Completer _onDispose = new Completer();
  Future get disposed => _onDispose.future;

  Disposable._([dynamic callback(), dynamic onData(dynamic value)])
    : this._callback = callback != null ? callback : Disposable.noop,
        this._onData = onData != null ? onData : Disposable.noop1;

  /*
   * Creates a disposable instance which triggers (if provided) the supplied callback, potentially with a completing value
   */
  factory Disposable([dynamic callback(), dynamic onData(dynamic value)]) {
    return new Disposable._(callback, onData);
  }

  /*
   * Triggers the cleanup callback provided to the disposable
   * Dispose cannot be called multiple times on the same instance
   */
  void dispose([dynamic value]) {
    if(_isDisposed) { return; }
    _isDisposed = true;
    _callback();
    if(value != null) {
      _onData(value);
    }
    _onDispose.complete(value);
  }

  /*
   * Triggers the cleanup callback provided to the disposable and passes along the error message
   * Dispose cannot be called multiple times on the same instance
   */
  void disposeError(dynamic error) {
    if(_isDisposed) { return; }
    _isDisposed = true;
    _callback(error);
    _onDispose.completeError(error);
  }
}