part of disposable;

/// Provides a method of encapsulating a series of actions which ensure proper disposal of
/// the disposable instance prior to exiting
void using(IDisposable disposable, UsingFunction func) {
  func(disposable);
  disposable.dispose();
}

/// Provides a method of encapsulating a series of actions which ensure proper disposal of
/// the disposable instance prior to exiting but following the completion of a Future value
/// which is expected to be emitted from the contained code block
Future usingAsync(IFutureDisposable disposable, UsingAsyncFunction func) {
  return func(disposable).then(
      (value) {
        disposable.dispose(value);
        return disposable.disposed;
      },
      onError: (error) {
        disposable.disposeError(error);
        return disposable.disposed;
      });
}